from ftw.contentnav.behaviors.content_categories import IContentCategories
from ftw.contentnav.interfaces import ICategorizable
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer


@indexer(ICategorizable)
def categories(obj, **kw):
    if IDexterityContent.providedBy(obj) and ICategorizable.providedBy(obj):
        return [item.encode('utf-8') for item in
                IContentCategories(obj).content_categories]

    return ()



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentnav.testing import FTW_CONTENTNAV_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase


class TestContentCategoriesBehavior(TestCase):

    layer = FTW_CONTENTNAV_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        # add sample fti
        self.fti = DexterityFTI('Sample')
        self.fti.schema = \
            'ftw.contentnav.tests.test_content_categorie_behavior.ISampleDX'
        self.fti.behaviors = (
            'ftw.contentnav.behaviors.content_categories.IContentCategories', )

        self.portal.portal_types._setObject('Sample', self.fti)

    def test_category_index(self):
        catalog = getToolByName(self.portal, 'portal_catalog')

        create(Builder('sample')
               .having(content_categories=(u'DEMO1', )))

        self.assertTrue(catalog({'get_content_categories': 'DEMO1'})[0])

    def test_category_index_umlauts(self):
        catalog = getToolByName(self.portal, 'portal_catalog')

        create(Builder('sample')
               .having(content_categories=(u'WITH unicode \xe4', )))

        unique_values = catalog.Indexes[
            'get_content_categories'].uniqueValues()

        self.assertIn("WITH unicode \xc3\xa4", unique_values)

    @browsing
    def test_adding_new_categories_only_for_managers(self, browser):
        content = create(Builder('sample')
                         .titled('Democontent'))

        user = create(Builder('user')
                      .with_roles('Site Administrator', on=content))

        browser.login(user.getId()).visit(content, view='@@edit')

        selector = \
            '#formfield-form-widgets-IContentCategories-new_content_categories'
        self.assertFalse(browser.css(selector),
                         'New categories field should no be visible.')

        browser.login().visit(content, view='@@edit')
        self.assertTrue(
            browser.css(selector),
            'New categories field should be visible.')

    @browsing
    def test_categories_from_other_contents_are_available(self, browser):
        categories = (u'Cat 1', u'Cat 2', 'Cat 3')
        create(Builder('sample')
               .titled('Democontent')
               .having(content_categories=categories))

        content = create(Builder('sample')
                         .titled('Democontent'))

        browser.login().visit(content, view='@@edit')

        selector = '#form-widgets-IContentCategories-' \
                   'content_categories input[type=checkbox]'

        self.assertEqual(
            3,
            len(browser.css(selector)),
            'Some categories are missing in the edit-form')



from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFCore.utils import getToolByName
from zope.component import queryUtility
from zope.interface import directlyProvides
from zope.schema import vocabulary
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm


def contentcategories_vocabulary(context):
    normalizer = queryUtility(IIDNormalizer)
    catalog = getToolByName(context, 'portal_catalog')
    terms = []
    for term in catalog.uniqueValuesFor("get_content_categories"):
        terms.append(
            SimpleTerm(
                value=term.decode('utf8'),
                token=normalizer.normalize(term.decode('utf8')),
                title=term.decode('utf8')))
    return vocabulary.SimpleVocabulary(terms)

directlyProvides(contentcategories_vocabulary, IVocabularyFactory)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentnav.testing import FTW_CONTENTNAV_FUNCTIONAL_TESTING
from plone.dexterity.fti import DexterityFTI
from Products.Five.browser import BrowserView as View
from unittest2 import TestCase
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class TestContentNavViewlet(TestCase):

    layer = FTW_CONTENTNAV_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        # add sample fti
        self.fti = DexterityFTI('Sample')
        self.fti.klass = 'plone.dexterity.content.Container'
        self.fti.schema = \
            'ftw.contentnav.tests.test_content_categorie_behavior.ISampleDX'
        self.fti.behaviors = (
            'ftw.contentnav.behaviors.content_categories.IContentCategories', )

        self.portal.portal_types._setObject('Sample', self.fti)

    def _get_viewlet(self, obj):
        view = View(obj, obj.REQUEST)
        manager_name = 'plone.belowcontentbody'
        manager = queryMultiAdapter(
            (obj, obj.REQUEST, view),
            IViewletManager,
            manager_name)

        self.failUnless(manager)

        # Set up viewlets
        manager.update()
        name = 'ftw.contentnav.contentnavlisting'
        return [v for v in manager.viewlets if v.__name__ == name]

    def test_viewlet_with_one_category(self):
        folder = create(Builder('sample'))
        content = create(Builder('sample')
                         .titled('Democontent')
                         .within(folder)
                         .having(content_categories=(u'WITH unicode \xe4', )))

        viewlet = self._get_viewlet(folder)[0]
        self.assertTrue(viewlet.available())

        self.assertIn(
            ('WITH unicode \xc3\xa4',
                [('Democontent', content.absolute_url(), '')]),
            viewlet.get_content())

    def test_viewlet_with_multiple_categories(self):
        folder = create(Builder('sample'))
        content = create(Builder('sample')
                         .titled('Democontent')
                         .within(folder)
                         .having(content_categories=(
                             u'WITH unicode \xe4', u'WITHOUT unicode')))

        viewlet = self._get_viewlet(folder)[0]
        self.assertTrue(viewlet.available())

        self.assertIn(
            ('WITH unicode \xc3\xa4',
                [('Democontent', content.absolute_url(), '')]),
            viewlet.get_content())

        self.assertIn(
            ('WITHOUT unicode',
                [('Democontent', content.absolute_url(), '')]),
            viewlet.get_content())

    def test_viewlet_without_categories(self):
        folder = create(Builder('sample'))
        create(Builder('sample')
               .titled('Democontent')
               .within(folder)
               .having(content_categories=(tuple())))

        viewlet = self._get_viewlet(folder)[0]
        self.assertIs(False, viewlet.available())

    def test_adding_new_category_using_the_new_categories_field(self):
        folder = create(Builder('sample'))
        content = create(Builder('sample')
                         .titled('Democontent')
                         .within(folder)
                         .having(new_content_categories=(u'WITH \xe4', )))

        viewlet = self._get_viewlet(folder)[0]

        self.assertTrue(viewlet.available())

        self.assertIn(
            ('WITH \xc3\xa4',
                [('Democontent', content.absolute_url(), '')]),
            viewlet.get_content())



from ftw.upgrade import UpgradeStep


class RenderContentNavInDifferentViewlet(UpgradeStep):
    """Render content nav in different viewlet.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder import registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_ID, setRoles
from plone.app.testing import TEST_USER_NAME
from plone.autoform.interfaces import IFormFieldProvider
from zope import schema
from zope.configuration import xmlconfig
from zope.interface import alsoProvides
from zope.interface import Interface


class FtwContentnavLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        import ftw.contentnav
        xmlconfig.file('configure.zcml', ftw.contentnav,
                       context=configurationContext)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.contentnav:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


class ISampleDX(Interface):
    title = schema.TextLine(
        title=u'Title',
        required=False)

alsoProvides(ISampleDX, IFormFieldProvider)


class SampleBuilder(DexterityBuilder):
    portal_type = 'Sample'

registry.builder_registry.register('sample', SampleBuilder)


FTW_CONTENTNAV_FIXTURE = FtwContentnavLayer()
FTW_CONTENTNAV_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_CONTENTNAV_FIXTURE,), name="FtwContentnav:Integration")
FTW_CONTENTNAV_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_CONTENTNAV_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwContentnav:Functional")



from ftw.contentnav.testing import FTW_CONTENTNAV_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase


class TestPackageInstall(TestCase):

    layer = FTW_CONTENTNAV_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi_tool = getToolByName(self.portal, 'portal_quickinstaller')

    def test_package_installed(self):
        pid = 'ftw.contentnav'
        installed = [p['id'] for p in self.qi_tool.listInstalledProducts()]
        self.assertIn(
            pid, installed, 'package appears not to have been installed')

    def test_index_get_content_categories_installed(self):
        catalog = getToolByName(self.portal, "portal_catalog")
        self.assertIn('get_content_categories', catalog.indexes())

    def test_installed(self):
        portal_setup = getToolByName(self.portal, 'portal_setup')
        version = portal_setup.getLastVersionForProfile('ftw.contentnav:default')
        self.assertNotEqual(version, None, 'Maybe the metadata.xml is missing.')
        self.assertNotEqual(version, 'unknown', 'Maybe the metadata.xml is missing.')
